@@ -1,22 +1,22 @@
 /*****
 *
-* Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2020 CS GROUP - France. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation; either version 2, or (at your option)
+* it under the terms of the GNU Lesser General Public License as published by
+* the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
+* GNU Lesser General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License along
+* You should have received a copy of the GNU Lesser General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
@@ -44,6 +44,15 @@
 
 
 
+#ifdef HAVE_PCRE
+# include <pcre.h>
+# define _CRITERION_REGEX_FLAGS 0
+# define _CRITERION_REGEX_FLAGS_CASELESS PCRE_CASELESS
+#else
+# define _CRITERION_REGEX_FLAGS REG_EXTENDED|REG_NOSUB
+# define _CRITERION_REGEX_FLAGS_CASELESS REG_ICASE
+#endif
+
 
 struct match_cb {
         unsigned int nmatch;
@@ -54,7 +63,11 @@ struct match_cb {
 
 
 struct regex_value {
+#ifdef HAVE_PCRE
+        pcre *regex;
+#else
         regex_t regex;
+#endif
         char *regex_string;
 };
 
@@ -364,11 +377,13 @@ static int regex_match(const idmef_criterion_value_t *cv, idmef_criterion_operat
         if ( ! str )
                 return 0;
 
-        ret = regexec(&rv->regex, str, 0, NULL, 0);
-        if ( operator & IDMEF_CRITERION_OPERATOR_NOT )
-                return (ret == REG_NOMATCH) ? 1 : 0;
-        else
-                return (ret != REG_NOMATCH) ? 1 : 0;
+#ifdef HAVE_PCRE
+        ret = (pcre_exec(rv->regex, NULL, str, strlen(str), 0, 0, NULL, 0) < 0) ? 0 : 1;
+#else
+        ret = (regexec(&rv->regex, str, 0, NULL, 0) == REG_NOMATCH) ? 0 : 1;
+#endif
+
+        return (operator & IDMEF_CRITERION_OPERATOR_NOT) ? !ret : ret;
 }
 
 
@@ -413,7 +428,11 @@ static void regex_destroy(idmef_criterion_value_t *cv)
         rv = cv->value;
 
         free(rv->regex_string);
+#ifdef HAVE_PCRE
+        pcre_free(rv->regex);
+#else
         regfree(&rv->regex);
+#endif
         free(rv);
 }
 
@@ -703,13 +722,13 @@ int idmef_criterion_value_new_broken_down_time(idmef_criterion_value_t **cv, con
 }
 
 
-
-
 int idmef_criterion_value_new_regex(idmef_criterion_value_t **cv, const char *regex, idmef_criterion_operator_t op)
 {
         int ret;
+        char errbuf[1024];
+        const char *errptr = errbuf;
         struct regex_value *rv;
-        int flags = REG_EXTENDED|REG_NOSUB;
+        int flags = _CRITERION_REGEX_FLAGS;
 
         ret = idmef_criterion_value_new(cv);
         if ( ret < 0 )
@@ -729,20 +748,24 @@ int idmef_criterion_value_new_regex(idmef_criterion_value_t **cv, const char *re
         }
 
         if ( op & IDMEF_CRITERION_OPERATOR_NOCASE )
-                flags |= REG_ICASE;
+                flags |= _CRITERION_REGEX_FLAGS_CASELESS;
 
+#ifdef HAVE_PCRE
+        int erroffset;
+
+        rv->regex = pcre_compile(rv->regex_string, flags, &errptr, &erroffset, NULL);
+        if ( ! rv->regex ) {
+#else
         ret = regcomp(&rv->regex, rv->regex_string, flags);
         if ( ret != 0 ) {
-                char errbuf[1024];
-
                 regerror(ret, &rv->regex, errbuf, sizeof(errbuf));
-
+#endif
                 free(rv->regex_string);
                 free(rv);
                 free(*cv);
 
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_CRITERION_INVALID_REGEX,
-                                             "error compiling regex: %s", errbuf);
+                                             "error compiling regex: %s", errptr);
         }
 
         (*cv)->match = regex_match;
