@@ -1,21 +1,21 @@
 /*****
 *
-* Copyright (C) 2002-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2020 CS GROUP - France. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation; either version 2, or (at your option)
+* it under the terms of the GNU Lesser General Public License as published by
+* the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
+* GNU Lesser General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License along
+* You should have received a copy of the GNU Lesser General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
@@ -99,6 +99,9 @@ double prelude_simple_strtod(const char *s, char **endptr);
 
 uint32_t prelude_crc32(const unsigned char *data, size_t length);
 
+int prelude_unicode_to_string(prelude_string_t *out, const char *in, size_t len);
+
+
 #ifdef __cplusplus
  }
 #endif
