@@ -1,21 +1,21 @@
 /*****
 *
-* Copyright (C) 2007-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2007-2020 CS GROUP - France. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation; either version 2, or (at your option)
+* it under the terms of the GNU Lesser General Public License as published by
+* the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
+* GNU Lesser General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License along
+* You should have received a copy of the GNU Lesser General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
@@ -484,7 +484,7 @@ int prelude_failover_save_msg(prelude_failover_t *failover, prelude_msg_t *msg)
 
         do {
                 ret = prelude_msg_write(msg, failover->wfd);
-        } while ( ret < 0 && errno == EINTR );
+        } while ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EINTR );
 
         if ( ret < 0 )
                 goto error;
