@@ -1,21 +1,21 @@
 /*****
 *
-* Copyright (C) 2001-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2020 CS GROUP - France. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation; either version 2, or (at your option)
+* it under the terms of the GNU Lesser General Public License as published by
+* the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
+* GNU Lesser General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License along
+* You should have received a copy of the GNU Lesser General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
@@ -1029,3 +1029,20 @@ const unsigned char *prelude_msg_get_message_data(prelude_msg_t *msg)
         return msg->payload + msg->fd_write_index;
 }
 
+
+
+int prelude_msg_clone(prelude_msg_t **dst, const prelude_msg_t *src)
+{
+        size_t size = sizeof(*src) + src->hdr.datalen;
+
+        *dst = malloc(size);
+        if ( ! *dst )
+                return prelude_error_from_errno(errno);
+
+        memcpy(*dst, src, size);
+
+        if ( src->payload )
+                (*dst)->payload = (unsigned char *) (*dst) + sizeof(**dst);
+
+        return 0;
+}
