@@ -36,17 +36,17 @@ int main(void)
         assert(idmef_message_new(&idmef) == 0);
 
         assert(idmef_message_set_string(idmef, "alert.classification.text", "Random value") == 0);
-        assert(idmef_message_get_string(idmef, "alert.classification.text", &res) == 0);
+        assert(idmef_message_get_string(idmef, "alert.classification.text", &res) > 0);
         assert(strcmp("Random value", res) == 0);
         free(res);
 
         assert(idmef_message_set_string(idmef, "alert.assessment.impact.severity", "high") == 0);
-        assert(idmef_message_get_string(idmef, "alert.assessment.impact.severity", &res) == 0);
+        assert(idmef_message_get_string(idmef, "alert.assessment.impact.severity", &res) > 0);
         assert(strcmp("high", res) == 0);
         free(res);
 
         assert(idmef_message_set_number(idmef, "alert.assessment.confidence.confidence", 10.2) == 0);
-        assert(idmef_message_get_number(idmef, "alert.assessment.confidence.confidence", &resd) == 0);
+        assert(idmef_message_get_number(idmef, "alert.assessment.confidence.confidence", &resd) == 1);
         assert(reldif(resd, 10.2) <= FLOAT_TOLERANCE);
 
         assert(idmef_message_set_string(idmef, "alert.assessment.impact.severity", "Random value") < 0);
